#
#   PROBE - Controls
#

from pygame import draw, Rect
from albow.widget import Widget
from albow import resource
from albow.resource import get_image, get_font
from albow.controls import TextField
from images import get_image_set
from vector_algebra import add

class ImageButton(Widget):

	font = get_font(10, "Vera.ttf")
	fg_color = (255, 255, 0)
	bg_color = (128, 128, 128)

	def __init__(self, title, image_name, pos_attr = None, pos_value = None,
			action = None, border = None):
		self.title = title
		self.image = get_image(image_name, border = border)
		r = self.image.get_rect()
		r.width += 2
		r.height += self.font.get_linesize() + 3
		if pos_attr:
			setattr(r, pos_attr, pos_value)
		Widget.__init__(self, r)
		self.action = action
	
	def draw(self, surf):
		color = self.fg_color
		frame = surf.get_rect()
		lbl = self.font.render(self.title, True, self.fg_color)
		lr = lbl.get_rect()
		lr.midtop = frame.midtop
		lr.top = 1
		y = lr.bottom + 1
		surf.blit(lbl, lr)
		surf.blit(self.image, (1, y + 1))
		draw.rect(surf, color, frame, 1)
		draw.line(surf, color, (frame.left, y), (frame.right, y))
	
	def mouse_down(self, e):
		self.action()

#---------------------------------------------------------------------------

class ChangingImageButton(Widget):

	font = get_font(10, "Vera.ttf")
	fg_color = (0, 0, 0)
	down = False
	
	def __init__(self, image_name, border = 0, title = None, action = None,
			font = None, fg_color = None):
		self.images = get_image_set(image_name, 2, 1, border = border)
		self.title = title
		if font:
			self.font = font
		if fg_color:
			self.fg_color = fg_color
		if action:
			self.action = action
		rect = Rect((0, 0), self.images.cell_size)
		if title:
			rect.height += self.font.get_height()
		Widget.__init__(self, rect)
	
	def draw(self, surf):
		frame = surf.get_rect()
		image = self.images[self.down]
		r = image.get_rect()
		r.bottomleft = frame.bottomleft
		surf.blit(image, r)
		title = self.title
		if title:
			buf = self.font.render(title, True, self.fg_color)
			r = buf.get_rect()
			r.midtop = frame.midtop
			surf.blit(buf, r)
	
	def mouse_down(self, e):
		self.down = True
	
	def mouse_drag(self, e):
		p = e.local
		r = Rect((0,0), self.rect.size)
		self.down = r.collidepoint(*p)
	
	def mouse_up(self, e):
		if self.down:
			self.call_handler('action')
		self.down = False

#---------------------------------------------------------------------------

class PushbuttonSwitch(Widget):

	font = get_font(10, "Vera.ttf")
	text_color = (0, 0, 0)
	depth = (4, 4)
	down_offset = (2, 2)

	def __init__(self, text, image_name, action = None):
		self.text = text
		self.images = get_image_set(image_name, 2, 1)
		Widget.__init__(self, Rect((0, 0), self.images.cell_size))
		if action:
			self.action = action
		self.down = 0
	
	def draw(self, surf):
		surf.blit(self.images[self.down], (0, 0))
		text = self.text
		if text:
			frame = surf.get_rect()
			depthx, depthy = self.depth
			frame.width -= depthx
			frame.height -= depthy
			if self.down:
				frame.move_ip(self.down_offset)
			buf = self.font.render(text, True, self.text_color)
			r = buf.get_rect()
			r.center = frame.center
			surf.blit(buf, r)
	
	def mouse_down(self, e):
		self.down = 1
		self.action()
	
	def mouse_up(self, e):
		self.down = 0

#---------------------------------------------------------------------------

class NumericField(TextField):

	font = None

	def __init__(self, max_digits = 8, max_dp = 0, signed = False, exp = False,
			font = None):
		font = font or self.font or resource.default_font
		self.max_digits = max_digits
		self.max_dp = max_dp
		self.signed = signed
		self.exp = exp
		n = self.calc_width()
		self.chars_wide = n
		width, height = font.size("0" * n)
		TextField.__init__(self, width = width, font = font)
	
	def calc_width(self):
		return self.signed + self.max_digits + (self.exp or self.max_dp > 0) + 3 * self.exp

	#def key_down(self, e):
	#	#print "NumericField.key_down:", e.key ###
	#	if 32 <= e.key <= 255:
	#	c = e.unicode.upper()
	
	def insert_char(self, c):
		text = self.text
		if c == "-":
			if "E" in text:
				mt, et = text.split("E")
				self.change_text(mt + "E" + flip_sign(et))
			elif self.signed:
				self.change_text(flip_sign(text))
			return
		if c == ".":
			if not ("." in text or "E" in text):
				self.change_text(text + c)
			return
		if c == "E":
			if self.exp:
				if c not in text:
					self.change_text(text + c)
			return
		if "0" <= c <= "9":
			ss, ds1, ds2, ms, ds3 = self.split_text()
			#print "NumericField.key_down: splat =", (ss, ds1, ds2, ms, ds3) ###
			if self.exp:
				if len(ds1) + len(ds2) == self.max_digits:
					return
			else:
				#print "...max_dp =", self.max_dp ###
				if "." in text:
					#print "...checking num dp:", len(ds2) ###
					if len(ds2) == self.max_dp:
						return
				else:
					#print "...checking num non dp:", len(ds1) ###
					if len(ds1) == self.max_digits - self.max_dp:
						#print "...max_dp reached" ###
						return
			if len(ds3) == 2:
				#print "...max exponent reached" ###
				return
		if c in "0123456789\x08\x7f":
			#print "NumericField.key_down: passing", e.key ###
			return TextField.insert_char(self, c)
	
	def split_text(self):
		s = self.text
		if self.signed:
			ms = " "
		else:
			ms = ""
		ds1 = ""
		ds2 = ""
		if self.exp:
			es = " "
		else:
			es = ""
		ds3 = ""
		if s.startswith("-"):
			ms, s = "-", s[1:]
		if "E" in s:
			s, ds3 = s.split("E")
			if ds3.startswith("-"):
				es, ds3 = "-", ds3[1:]
		if "." in s:
			ds1, ds2 = s.split(".")
		else:
			ds1, ds2 = s, ""
		return ms, ds1, ds2, es, ds3
	
	def get_value(self):
		text = self.get_text()
		if not text:
			return None
		try:
			val = float(text)
		except ValueError:
			return 0.0
		return val

def flip_sign(text):
	if text.startswith("-"):
		return text[1:]
	else:
		return "-" + text

#---------------------------------------------------------------------------

class LedField(NumericField):

	font = get_font(12, "VeraMono.ttf")
	fg_color = (255, 0, 0)
	editing = False
	
	def draw(self, surf):
		#print "LedField.draw:", surf.get_rect() ###
		text = self.format_text()
		#print "...formatted text =", repr(text) ###
		buf = self.font.render(text, True, self.fg_color)
		surf.blit(buf, (0, 0))

	def format_text(self):
		#editing = self.has_focus()
		editing = self.editing
		if not editing and not self.text:
			return "-" * self.chars_wide
		if editing:
			fill1, fill2 = "_", "_"
		else:
			fill1, fill2 = " ", "0"
		ms, ds1, ds2, es, ds3 = self.split_text()
		#print "LedField.format_text: splat =", (ms, ds1, ds2, es, ds3) ###
		nd1 = self.max_digits - self.max_dp
		s1 = ("%*s" % (nd1, ds1)).replace(" ", fill1)
		#print "...s1 =", repr(s1) ###
		s2 = ""
		s3 = ""
		if self.exp:
			if ds2:
				s2 = "." + ds2
		else:
			if self.max_dp > 0:
				if editing and "." not in self.text:
					dp = " "
				else:
					dp = "."
				s2 = dp + ("%-*s" % (self.max_dp, ds2)).replace(" ", fill2)
			#print "..s2 =", repr(s2) ###
		if "E" in self.text:
			s3 = " " + ("%2s" % ds3).replace(" ", fill1)
		return ms + s1 + s2 + es + s3

#---------------------------------------------------------------------------

class Bezel(Widget):

	border_width = 3
	bg_color = (0, 0, 0)

	def __init__(self, client):
		w = self.border_width
		rect = client.rect.inflate((2 * w, 2 * w))
		rect.topleft = add(client.rect.topleft, (-w, -w))
		client.rect.topleft = (w, w)
		Widget.__init__(self, rect)
		self.add(client)
