#
#   PROBE - Data structure representing a probe design
#

import vehicle
import instruments

R = 10 # Instrument range increase factor for testing

class DesignElement(object):
	#  kind         string
	#  model_name   string
	#  mass         float, kg
	#  constructor  class or function
	#  params       dict of params for the constructor
	#  cost         number of dollars it costs to build
	
	mass_name = "Mass"

	def __init__(self, kind, model_name, mass, cost, constructor, **params):
		self.kind = kind
		self.model_name = model_name
		self.mass = mass
		self.constructor = constructor
		self.cost = cost
		self.params = params
	
	def total_mass(self):
		return self.mass
	
	def total_cost(self):
		return self.cost
	
	def get_specifications(self, result):
		result.append((self.mass_name, "%.2e kg" % self.mass))
		result.append(("Cost", "$%s" % self.cost))

	def build(self):
		return self.constructor(**self.params)

#---------------------------------------------------------------------------

class InstrumentDesign(DesignElement):

	range = None
	accuracy = None
	sensitivity = None
	sensitivity_format = None
	
	def __init__(self, *args, **kwds):
		self.range = kwds.get('range')
		self.accuracy = kwds.get('accuracy')
		self.sensitivity = kwds.get('sensitivity')
		self.sensitivity_format = kwds.pop('sensitivity_format', None)
		DesignElement.__init__(self, *args, **kwds)

	def add_to_design(self, des):
		if self not in des.instruments:
			des.instruments.append(self)

	def remove_from_design(self, des):
		if self in des.instruments:
			des.instruments.remove(self)

	def get_specifications(self, result):
		if self.range is not None:
			result.append(("Range", "%.2e km" % (1e-3 * self.range)))
		if self.accuracy is not None:
			result.append(("Accuracy", "%s%%" % (100 * self.accuracy)))
		if self.sensitivity is not None:
			format, scale = self.sensitivity_format
			result.append(("Sensitivity", format % (scale * self.sensitivity)))
		DesignElement.get_specifications(self, result)

#---------------------------------------------------------------------------

class BoosterStageDesign(DesignElement):
	#  Isp_sec        float, s
	#  burn_rate      float, kg/s
	#  fuel_capacity  float, kg

	mass_name = "Empty mass"

	def __init__(self, *args, **kwds):
		self.Isp_sec = kwds.get('Isp_sec')
		self.burn_rate = kwds.get('burn_rate')
		self.fuel_capacity = kwds.get('fuel_capacity')
		DesignElement.__init__(self, *args, **kwds)

	def add_to_design(self, des):
		if self not in des.booster_stages:
			des.booster_stages.append(self)

	def remove_from_design(self, des):
		if self in des.booster_stages:
			des.booster_stages.remove(self)

	def get_specifications(self, result):
		Isp_sec = self.Isp_sec
		Isp = 9.8 * Isp_sec
		b = self.burn_rate
		F = Isp * b
		result.append(("Isp", "%s Ns/kg (%s s)" % (Isp, Isp_sec)))
		result.append(("Burn rate", "%.2e kg/s" % b))
		result.append(("Thrust", "%.2e N" % F))
		result.append(("Fuel capacity", "%.2e kg" % self.fuel_capacity))
		DesignElement.get_specifications(self, result)

#---------------------------------------------------------------------------

class BasicVehicleDesign(BoosterStageDesign):
	
	def add_to_design(self, des):
		des.basic_vehicle = self
	
	def remove_from_design(self, des):
		if des.basic_vehicle is self:
			des.basic_vehicle = None

#---------------------------------------------------------------------------

class DesignError(StandardError):
	pass

#---------------------------------------------------------------------------

class ProbeDesign(object):
	#  mission_name       string
	#  basic_vehicle      BasicVehicleDesign
	#  instruments        [InstrumentDesign]
	#  booster_stages     [BoosterStageDesign]  last is first to burn
	#  selected_element   DesignElement
	
	mission_name = ""
	basic_vehicle = None
	#selected_category = None
	selected_element = None
	
	def __init__(self):
		self.instruments = []
		self.booster_stages = []
	
	def total_mass(self):
		mass = 0
		if self.basic_vehicle:
			mass += self.basic_vehicle.total_mass()
		for elem in self.instruments:
			mass += elem.total_mass()
		for elem in self.booster_stages:
			mass += elem.total_mass()
		return mass
	
	def total_cost(self):
		cost = 0
		if self.basic_vehicle:
			cost += self.basic_vehicle.total_cost()
		for elem in self.instruments:
			cost += elem.total_cost()
		for elem in self.booster_stages:
			cost += elem.total_cost()
		return cost
	
	def check_design(self, game):
		if not self.basic_vehicle:
			raise DesignError("We're not going to get far without a basic vehicle.")
		if not self.instruments:
			raise DesignError("There won't be much point in going if we don't "
				"have at least one instrument on board.")
		if not self.booster_stages:
			raise DesignError("It's going to be a bit difficult lifting off without "
				"a booster.")
		cost = self.total_cost()
		print "ProbeDesign.check_design: cost =", cost ###
		if cost > game.budget:
			raise DesignError("We can't afford to build that -- it'll cost $%s, and "
				"we've only got a budget of $%s." % (cost, game.budget))
	
	def build_vehicle(self, game):
		v = vehicle.Vehicle(self.mission_name)
		v.add_stage(self.basic_vehicle.build())
		for inst in self.instruments:
			v.add_instrument(inst.build())
		for stage in self.booster_stages:
			v.add_stage(stage.build())
		v.name_stages()
		game.budget -= self.total_cost()
		return v
	
#---------------------------------------------------------------------------

basic_vehicles = [
	BasicVehicleDesign("Basic Vehicle", "PBV-1", 100, 200000,
		vehicle.VehicleStage, empty_mass = 100, Isp_sec = 300, burn_rate = 1,
		fuel_capacity = 1000),
]

instruments = [
	InstrumentDesign("Temperature sensor", "THRM-1", 10, 100000,
		instruments.TemperatureSensor, range = R * 1e9, accuracy = 0.2),
	InstrumentDesign("Atmospheric pressure sensor", "BAR-1", 10, 100000,
		instruments.AtmosphericPressureSensor, range = R * 1e9, accuracy = 0.2),
	InstrumentDesign("Spectrometer", "SPCT-1", 50,  100000,
		instruments.Spectrometer, range = R * 1e9, accuracy = 0.2, sensitivity = 0.05,
		sensitivity_format = ("%s%%", 100)),
]

booster_stages = [
	BoosterStageDesign("Booster", "ATLAS-17", 1000, 300000,
		vehicle.VehicleStage, empty_mass = 1000, Isp_sec = 350, burn_rate = 10e3,
		fuel_capacity = 500000),
]

#---------------------------------------------------------------------------

class DesignElementCategory(object):
	#  name        string
	#  items       [DesignElement]
	#  selection   int or None
	
	selection = None
	
	def __init__(self, name, items):
		self.name = name
		self.items = items
	
	def num_items(self):
		return len(self.items)
	
	def selected_item(self):
		i = self.selection
		if i is None:
			return None
		else:
			return self.items[i]

category_items = [
	DesignElementCategory("Basic Vehicle", basic_vehicles),
	DesignElementCategory("Instruments", instruments),
	DesignElementCategory("Booster Stages", booster_stages),
]

categories = DesignElementCategory("", category_items)

def selected_element():
	cat = categories.selected_item()
	if cat:
		return cat.selected_item()

#def num_categories():
#	return len(categories.items)
#
#def category_no(i):
#	return categories.items[i]
#
#def selected_category_no():
#	return categories.selection
#
#def selected_category():
#	return categories.selected_item()
#
#def selected_element_no():
#	cat = self.selected_category()
#	if cat:
#		return cat.selection
#	else:
#		return None

def create_test_design():
	des = ProbeDesign()
	des.mission_name = "Qwerty 1"
	des.basic_vehicle = basic_vehicles[0]
	des.instruments = [instruments[0], instruments[2]]
	des.booster_stages = [booster_stages[0]]
	return des
