#
#   PROBE - Mission control screen
#

from math import floor
from pygame import Rect, draw
from pygame.locals import * #K_LEFT, K_RIGHT, K_TAB
from albow.widget import Widget
from albow.controls import Label
from albow.resource import get_font, get_image
from vector_algebra import add, subtract
from controls import ImageButton, PushbuttonSwitch, LedField, Bezel
from screen import ProbeScreen
from system_display import SystemDisplay
from astro_data_display import AstroDataDisplay
from tracking_display import TrackingDisplay
from sci_data_display import SciDataDisplay
import navigation

blue_grey = (0, 134, 152)

class MissionControlScreen(ProbeScreen):

	def __init__(self, shell):
		ProbeScreen.__init__(self, shell)
		game = shell.game
		frame = self.rect
		sd_size = frame.height - 200
		sd_rect = Rect(0, 0, sd_size, sd_size)
		co_rect = Rect(0, sd_size, 600, 200)
		cl_rect = Rect(0, 0, 200, 100)
		cl_rect.bottomleft =co_rect.bottomright
		add_rect = Rect(0, 0, 200, 200)
		add_rect.bottomleft = cl_rect.topleft
		ts_rect = Rect(0, 0, 200, 20)
		ts_rect.bottomleft = add_rect.topleft
		sdd_rect = Rect(600, 0, 200, ts_rect.top)
		self.system_display = SystemDisplay(sd_rect, game, paper = False)
		self.console = MissionControlConsole(co_rect, game)
		self.data_display = MissionAstroDataDisplay(add_rect, game)
		self.clock_and_launch_panel = ClockAndLaunchPanel(cl_rect, game)
		self.timescale_control = TimeScaleControl(ts_rect, game)
		self.sci_data_display = SciDataDisplay(sdd_rect, game)
		self.add(self.system_display)
		self.add(self.console)
		self.add(self.data_display)
		self.add(self.clock_and_launch_panel)
		self.add(self.timescale_control)
		self.add(self.sci_data_display)
		#lp_button = ImageButton("LAUNCH PREP", "launchpad_thumb.png",
		#	action = self.go_to_launch_prep)
		lp_button = navigation.launch_prep_button(self)
		lp_button.rect.topright = add(sdd_rect.topleft, (-10, 10))
		self.add(lp_button)
	
	def key_down(self, e):
		key = e.key
		#print "MissionControlScreen.key_down:", key ###
		if key == K_LEFT:
			self.game.decrease_timescale()
		elif key == K_RIGHT:
			self.game.increase_timescale()
		elif key == K_UP or key == K_DOWN:
			self.system_display.handle_key(e)
		elif key == K_TAB:
			self.game.set_timescale_number(0)
		elif key == K_a:
			self.keyboard_function_key("ATTD")
		elif key == K_b:
			self.keyboard_function_key("BURN")
		elif key == K_e:
			self.keyboard_function_key("EXEC")
		elif key == K_BACKSPACE or key == K_DELETE:
			self.keyboard_number_key("\x08")
		elif key == K_RETURN or key == K_KP_ENTER:
			self.keyboard_number_key("\x0d")
		elif K_KP0 <= key <= K_KP9:
			c = chr(48 + key - K_KP0)
			#print "...c =", repr(c) ###
			self.keyboard_number_key(c)
		elif key <= 255 and e.unicode in "0123456789.-Ee\x15":
			c = e.unicode
			#print "...keypad", repr(c) ###
			self.keyboard_number_key(c)
		else:
			ProbeScreen.key_down(self, e)
	
	def key_up(self, e):
		self.console.number_pad.keyboard_deactivate()
		self.console.function_pad.keyboard_deactivate()
		ProbeScreen.key_up(self, e)

	def keyboard_number_key(self, code):
		self.console.number_pad.keyboard_activate(code)
	
	def keyboard_function_key(self, code):
		self.console.function_pad.keyboard_activate(code)
	
#---------------------------------------------------------------------------

class MissionControlConsole(Widget):

	#bg_color = blue_grey
	monitor_image = get_image("monitor-200x200.jpg")
	panel_image = get_image("panel-200x200.jpg")
	
	current_command_field = None

	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		self.game = game
		self.add(TrackingDisplay(Rect(200, 0, 200, 200), game))
		self.attitude_field = LedField(4, 1, signed = True)
		self.attitude_field.rect.topleft = (420, 35)
		self.burn_time_field = LedField(5, 1, signed = False)
		self.burn_time_field.rect.topleft = add(self.attitude_field.rect.topright, (50, 0))
		self.number_pad = NumberPad(action = self.number_key_pressed)
		self.number_pad.rect.bottomleft = (415, 185)
		self.function_pad = FunctionPad(action = self.function_key_pressed)
		self.function_pad.rect.bottomleft = add(self.number_pad.rect.bottomright, (20, 0))
		self.add(Bezel(self.attitude_field))
		self.add(Bezel(self.burn_time_field))
		self.add(self.number_pad)
		self.add(self.function_pad)
		label_above(self, self.attitude_field.parent, "ATTITUDE")
		label_to_right(self, self.attitude_field.parent, "DEG")
		label_above(self, self.burn_time_field.parent, "BURN TIME")
		label_to_right(self, self.burn_time_field.parent, "SEC")
	
	def select_command_field(self, field):
		old_field = self.current_command_field
		if old_field:
			old_field.editing = False
		self.current_command_field = field
		if field:
			field.set_text("")
			field.editing = True
	
	def number_key_pressed(self, code):
		field = self.current_command_field
		if field:
			if code == "\x0d":
				field.editing = False
			else:
				field.insert_char(code)
	
	def function_key_pressed(self, code):
		if code == "ATTD":
			self.select_command_field(self.attitude_field)
		elif code == "BURN":
			self.select_command_field(self.burn_time_field)
		elif code == "EXEC":
			self.execute_command()
	
	def execute_command(self):
		self.select_command_field(None)
		attitude = self.attitude_field.get_value()
		burn_time = self.burn_time_field.get_value()
		print "EXEC: ATTD %s BURN %s" % (attitude, burn_time)
		system = self.game.current_system
		if system:
			vehicle = system.vehicle
			if vehicle:
				vehicle.manoeuvre(attitude, burn_time)

	def draw(self, surf):
		surf.blit(self.monitor_image, (0, 0))
		surf.blit(self.monitor_image, (200, 0))
		surf.blit(self.panel_image, (400, 0))
	
#---------------------------------------------------------------------------

class MissionClockDisplay(Widget):

	font = get_font(15, "VeraMono.ttf")
	fg_color = (255, 0, 0)
	bg_color = (0, 0, 0)
	
	def __init__(self, game):
		Widget.__init__(self, Rect(0, 0, 140, 20))
		self.game = game
	
	def draw(self, surf):
		time = int(floor(self.game.mission_clock))
		sign = " "
		if time < 0:
			sign = "-"
			time = -time
		time, secs = divmod(time, 60)
		time, mins = divmod(time, 60)
		days, hours = divmod(time, 24)
		text = "%s%05d %02d %02d %02d" % (sign, days, hours, mins, secs)
		buf = self.font.render(text, True, self.fg_color)
		r = buf.get_rect()
		r.center = surf.get_rect().center
		surf.blit(buf, r)

#---------------------------------------------------------------------------

class MissionAstroDataDisplay(AstroDataDisplay):

	#bg_image = None
	monitor_image = get_image("monitor-200x200.jpg")
	bg_color = blue_grey
	text_color = (0, 255, 0)
	left_margin = 20

	def draw(self, surf):
		surf.blit(self.monitor_image, (0, 0))
		AstroDataDisplay.draw(self, surf)

#---------------------------------------------------------------------------

class ClockAndLaunchPanel(Widget):

	bg_image = get_image("panel-200x100.jpg")
	
	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		mission_clock = MissionClockDisplay(game)
		mission_clock.rect.midtop = (100, 20)
		launch_button = PushbuttonSwitch("LAUNCH", "button-60x25.png", game.launch)
		launch_button.rect.topright = add(mission_clock.rect.bottomright, (0, 20))
		self.add(mission_clock)
		self.add(launch_button)
		label_above(self, mission_clock, "MISSION ELAPSED TIME", 'm')
		label_below(self, mission_clock, "       DAYS       HRS   MIN   SEC", 's')
	
	def draw(self, surf):
		surf.blit(self.bg_image, (0, 0))
	
	#def launch(self):
	#	self.game.launch()

#---------------------------------------------------------------------------

class TimeScaleControl(Widget):

	fg_color = (255, 255, 0)

	def __init__(self, rect, game):
		Widget.__init__(self, rect)
		self.game = game
	
	def draw(self, surf):
		frame = surf.get_rect()
		color = self.fg_color
		game = self.game
		draw.rect(surf, color, frame, 1)
		n = game.num_timescales()
		i = game.current_timescale_number()
		w = frame.width // n
		x = i * frame.width // n
		r = Rect(x, 0, w, frame.height)
		r.inflate_ip(0, -4)
		draw.rect(surf, color, r)
	
	def mouse_down(self, e):
		x, y = e.local
		game = self.game
		n = game.num_timescales()
		i = x * n // self.rect.width
		game.set_timescale_number(i)

#---------------------------------------------------------------------------

#class CommandDisplay(Widget):
#
#	def __init__(self):
#		Widget.__init__(self, Rect(0, 0, 100, 100))
#		self.attitude_field = LedField(4, 1, signed = True)
#		self.burn_time_field = LedField(5, 1, signed = False)
#		self.add_with_bezel(self.attitude_field, (0, 0))
#		self.add_with_bezel(self.burn_time_field, (0, 30))
#		self.shrink_wrap()
#	
#	def add_with_bezel(self, ctrl, pos):
#		bezel = Bezel(ctrl)
#		bezel.rect.topleft = pos
#		self.add(bezel)

#---------------------------------------------------------------------------

class Keypad(Widget):

	def __init__(self, action = None):
		Widget.__init__(self)
		self.switch_for_code = {}
		for i, row in enumerate(self.captions):
			for j, caption in enumerate(row):
				code = self.codes[i][j]
				switch = KeypadSwitch(self, i, j, caption)
				self.switch_for_code[code] = switch
				self.add(switch)
		self.shrink_wrap()
		if action:
			self.action = action
	
	def key_pressed(self, switch):
		row, col = switch.coords
		self.action(self.codes[row][col])
	
	def keyboard_activate(self, code):
		switch = self.switch_for_code.get(code)
		if switch:
			switch.down = True
			self.key_pressed(switch)
	
	def keyboard_deactivate(self):
		for switch in self.subwidgets:
			switch.down = False
	
	def action(self, code):
		pass

class KeypadSwitch(PushbuttonSwitch):

	def __init__(self, keypad, row, col, caption, image_name = None):
		PushbuttonSwitch.__init__(self, caption, image_name or keypad.image_name)
		self.keypad = keypad
		self.coords = row, col
		r = self.rect
		dw, dh = self.depth
		r.left = col * (r.width - dw)
		r.top = row * (r.height - dh)
	
	def action(self):
		self.keypad.key_pressed(self)

class NumberPad(Keypad):

	image_name = "button-25x25.png"
	
	captions = [
		["7", "8", "9", "C"],
		["4", "5", "6", "<-"],
		["1", "2", "3", "+/-"],
		["0", "."],
	]

	codes = [
		["7", "8", "9", "\x15"],
		["4", "5", "6", "\x08"],
		["1", "2", "3", "-"],
		["0", ".", "\x0d"],
	]

	def __init__(self, *args, **kwds):
		Keypad.__init__(self, *args, **kwds)
		enter_switch = KeypadSwitch(self, 3, 2, "ENTER", "button-50x25.png")
		enter_switch.rect.bottomright = self.rect.size
		self.switch_for_code["\x0d"] = enter_switch
		self.add(enter_switch)

class FunctionPad(Keypad):

	image_name = "button-45x25.png"
	
	captions = [
		["ATTD"],
		["BURN"],
		[],
		["EXEC"],
	]
	
	codes = captions

#---------------------------------------------------------------------------

label_font = get_font(10, "Vera.ttf")
small_label_font = get_font(9, "Vera.ttf")
label_color = (25, 25, 25)

label_fonts = dict(s = small_label_font, m = label_font)

def label_above(parent, field, text, size = 'm', dy = 2):
	label = make_label(parent, field, text, size)
	label.rect.midbottom = add(field.rect.midtop, (0, -dy))

def label_below(parent, field, text, size = 'm', dy = 2):
	label = make_label(parent, field, text, size)
	label.rect.midtop = add(field.rect.midbottom, (0, dy))

def label_to_right(parent, field, text, size = 'm', dx = 5):
	label = make_label(parent, field, text, size)
	label.rect.midleft = add(field.rect.midright, (dx, 0))

def make_label(parent, field, text, size = 'm'):
	font = label_fonts[size]
	label = Label(text, font = font)
	label.fg_color = label_color
	parent.add(label)
	return label
